#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <poll.h>

#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include "libsocketcan.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

typedef struct 
{
    const char *name;
    int fd;
    int bitrate;
    pthread_t rec_tid;
    int log_fd;
} can_block_t;


void *can_rec_thread(void *para)
{
    can_block_t *block = para;
    struct can_frame rx_frame;
    int nbytes, err;

    struct pollfd fds[] = {
        {
            .fd	= block->fd,
            .events	= POLLIN,
        },
    };

    while(1)
    {
        err = poll(fds, 1, 10000);
        if (err == -1)
        {
            printf("[%u]CAN POLL ERROR!\n", (uint32_t)time(NULL));
        }
        else if (err == 0)
        {
            /* timeout */
            continue;
        }
        else
        {
            nbytes = read(block->fd, &rx_frame, sizeof(rx_frame));
            if (nbytes != sizeof(rx_frame))
            {
                printf("[%u]CAN REC ERROR: %d!\n", (uint32_t)time(NULL), nbytes);
                continue;
            }
            else
            {
                if (nbytes != write(block->log_fd, &rx_frame, sizeof(rx_frame)))
                {
                    printf("CAN LOG ERROR: %d!\n", nbytes);
                }
            }
        }
    }
    return NULL;
}


can_block_t * can_create(const char *name, int bitrate;)
{
    char log_file_name[32];

    struct ifreq ifr;
    struct sockaddr_can addr;
    can_block_t *block;

    int s, ret;
    int family = PF_CAN, type = SOCK_RAW|SOCK_NONBLOCK, proto = CAN_RAW;

    block = malloc(sizeof(can_block_t));
    block->name = name;
    block->bitrate = bitrate;

    can_set_bitrate(block->name, block->bitrate);
    int state = 0;
    can_get_state(block->name,&state);
    if(state != CAN_STATE_STOPPED) {
        can_do_stop(block->name);
    }
    can_do_start(block->name);

    strcpy(ifr.ifr_name, block->name);
    s = socket(family, type, proto);
    if (s < 0)
    {
        printf("SOCKET ERROR\n");
        return NULL;
    }

    addr.can_family = family;
    if (ioctl(s, SIOCGIFINDEX, &ifr) < 0)
    {
        printf("SOCKET IOCTL ERROR\n");
        return NULL;
    }

    addr.can_ifindex = ifr.ifr_ifindex;
    if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0)
    {
        printf("SOCKET BIND ERROR");
        return NULL;
    }

    block->fd = s;

    ret = pthread_create(&block->rec_tid, NULL, can_rec_thread, block);
    if (ret)
    {
        printf("pthread_create can rec err!\n");
        return NULL;
    }
    memset(log_file_name, 0, sizeof(log_file_name));
    sprintf(log_file_name, "/mnt/%s_log.bin", block->name);

    block->log_fd = open(log_file_name, O_CREAT|O_WRONLY|O_TRUNC);
    if (block->log_fd < 0)
    {
        printf("open file %s error!\n", log_file_name);
        return NULL;
    }
    return block;
}
